=== modified file 'NEWS'
--- NEWS	2008-05-24 11:41:24 +0000
+++ NEWS	2008-05-27 01:05:57 +0000
@@ -52,6 +52,11 @@
       attribute '_remote_is_at_least_1_2' attribute errors.
       (Andrew Bennetts, #220806)
 
+    * Failure to delete an obsolete pack file should just give a warning
+      message, not a fatal error.  It may for example fail if the file is still
+      in use by another process.
+      (Martin Pool)
+      
     * Fix MemoryError during large fetches over HTTP by limiting the amount of
       data we try to read per ``recv`` call.  The problem was observed with
       Windows and a proxy, but might affect other environments as well.

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2008-05-22 05:48:22 +0000
+++ bzrlib/repofmt/pack_repo.py	2008-05-27 01:05:57 +0000
@@ -1589,10 +1589,8 @@
                 mode=self.repo.bzrdir._get_file_mode())
             # move the baseline forward
             self._packs_at_load = disk_nodes
-            # now clear out the obsolete packs directory
             if clear_obsolete_packs:
-                self.transport.clone('obsolete_packs').delete_multi(
-                    self.transport.list_dir('obsolete_packs'))
+                self._clear_obsolete_packs()
         finally:
             self._unlock_names()
         # synchronise the memory packs list with what we just wrote:
@@ -1624,6 +1622,16 @@
                 self._names[name] = sizes
                 self.get_pack_by_name(name)
 
+    def _clear_obsolete_packs(self):
+        """Delete everything from the obsolete-packs directory.
+        """
+        obsolete_pack_transport = self.transport.clone('obsolete_packs')
+        for filename in obsolete_pack_transport.list_dir('.'):
+            try:
+                obsolete_pack_transport.delete(filename)
+            except (errors.PathError, errors.TransportError), e:
+                warning("couldn't delete obsolete pack, skipping it:\n%s" % (e,))
+
     def _start_write_group(self):
         # Do not permit preparation for writing if we're not in a 'write lock'.
         if not self.repo.is_write_locked():

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2008-04-26 16:34:47 +0000
+++ bzrlib/tests/test_repository.py	2008-05-26 03:34:59 +0000
@@ -39,9 +39,13 @@
 from bzrlib.tests import (
     TestCase,
     TestCaseWithTransport,
+    TestSkipped,
     test_knit,
     )
-from bzrlib.transport import get_transport
+from bzrlib.transport import (
+    fakenfs,
+    get_transport,
+    )
 from bzrlib.transport.memory import MemoryServer
 from bzrlib.util import bencode
 from bzrlib import (
@@ -946,6 +950,22 @@
         pack_names = [node[1][0] for node in index.iter_all_entries()]
         self.assertTrue(large_pack_name in pack_names)
 
+    def test_fail_obsolete_deletion(self):
+        # failing to delete obsolete packs is not fatal
+        format = self.get_format()
+        server = fakenfs.FakeNFSServer()
+        server.setUp()
+        self.addCleanup(server.tearDown)
+        transport = get_transport(server.get_url())
+        bzrdir = self.get_format().initialize_on_transport(transport)
+        repo = bzrdir.create_repository()
+        repo_transport = bzrdir.get_repository_transport(None)
+        self.assertTrue(repo_transport.has('obsolete_packs'))
+        # these files are in use by another client and typically can't be deleted
+        repo_transport.put_bytes('obsolete_packs/.nfsblahblah', 'contents')
+        repo._pack_collection._clear_obsolete_packs()
+        self.assertTrue(repo_transport.has('obsolete_packs/.nfsblahblah'))
+
     def test_pack_after_two_commits_packs_everything(self):
         format = self.get_format()
         tree = self.make_branch_and_tree('.', format=format)

=== modified file 'bzrlib/transport/fakenfs.py'
--- bzrlib/transport/fakenfs.py	2006-07-28 16:05:23 +0000
+++ bzrlib/transport/fakenfs.py	2008-05-26 03:34:59 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,10 @@
 
 from stat import S_ISDIR
 
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    urlutils,
+    )
 from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
 
 
@@ -53,6 +56,11 @@
             else:
                 raise
 
+    def delete(self, relpath):
+        if urlutils.basename(relpath).startswith('.nfs'):
+            raise errors.ResourceBusy(self.abspath(relpath))
+        return self._decorated.delete(relpath)
+
 
 class FakeNFSServer(DecoratorServer):
     """Server for the FakeNFSTransportDecorator for testing with."""

